#!/bin/sh

#################################################################
# nom			: backup_config.sh
# description	: Scrit de persistance de l'installation de l'agent DSR sous ESX
# version		: 1.08
# date			: 27/09/2013 16:30
# autheur		: remi
#################################################################



# LISTE DES CODES ERREUR POUR backup_config.sh
#  1 "Echec de création du chemin temporaire $TMPDIR"
#  2 "Echec de création de la structure de beemo.tgz"
#  3 "Echec de création de beemo.tgz"
#  4 "Echec de la copie de beemo.tgz dans /bootbank"
#  5 "Echec de l'ajout du module beemo.tgz à boot.cfg"
# 51 "Echec de l'ajout du module beemo.tgz à boot.cfg. Impossible de vérifier la présence antérieure du module."
#  6 "Echec de la suppression du repertoire temporaire :\n $TMPDIR\nVous pouvez le supprimer manuellement, sinon il dera détruit au prochain reboot."
#  7 "Impossible d'inclure $FILE lors de la création de la structure de beemo.tgz"
#  8 "Licence non null dans fichier config mais fichier /usr/local/dsr/conf/$LICENCE/pub n'a pu être inclus à beemo.tgz"
# 99 "Vous devez être root pour lancer ce script."



# ****************************************
# rollbackInstall
# Descritpion :
#	Cette fonction annule les opérations effectuées jusqu'au point d'une erreur dans l'installation en se basant
#	sur le paramètre de niveau de rollback
# Paramètres :
#	1 : est le niveau de rollback à effectuer. Cette variable est une chaine. A chaque opération correspond à un caractère.
#	    Si ce caractère est présent dans la chaine c'est que cette opération a été effectuée et doit être annulée.
#	2 : le chemin TMPDIR
# ****************************************
rollbackInstall ()
{
	if [ "$1" ]; then
		echo "Annulation de l'installation... "
	fi
	# restore /bootbank/beemo.tgz avec /bootbank/boot.cfg.beemoinstallbak
	if [ `echo "$1" | grep c` ]; then
		mv /bootbank/boot.cfg.beemoinstallbak /bootbank/beemo.tgz
	fi
	# supprime /bootbank/beemo.tgz
	if [ `echo "$1" | grep b` ]; then
		rm /bootbank/beemo.tgz
	fi
	# supprime $TMPDIR
	if [ `echo "$1" | grep a` ]; then
		rm -Rf $2 
	fi
	if [ "$1" ]; then
		echo "Terminé"
	fi
}

# ****************************************
# handleError
# Descritpion :
#	Cette fonction sert à gérer le cas d'une erreur ou d'un warning
#	Elle vérifie si la commande précédente a fonctionné correctement.
#	Si elle a renctré un erreur, elle affiche un code d'erreur ainsi qu'un message d'erreur. 
#		Dans le cas d'une erreur, elle déclenche un rollback puis arrête le script avec le code d'erreur spécifié.
#		Dans le cas d'un warning, le script continue.
# Paramètres :
#	1 : est une chaine contenant "error" ou "warning" pour déterminer le niveau d'erreur
#	2 : est un code d'erreur
#	3 : est le message d'erreur à afficher
#	4 : est le niveau de rollback à effectuer
#	5 : est vide si l'on veut que la detection de l'erreur se base sur $?
#           est "force" si l'on forcer la détection d'une erreur
#	6 : est le chemin init_dir (pour passer à rollbackInstall ())
# ****************************************
handleError ()
{
	if [ $? -ne 0 ] || [ "$5" == "force" ]; then
		if [ "$1" == "error" ]; then
			echo -e "Erreur (code $2): $3"
			rollbackInstall $4 $6
			exit $2
		elif [ "$1" == "warning" ]; then
			echo -e "Warning (code $2): $3"
		else
			echo "Type d'erreur non supporté"
		fi
	fi
}

# ****************************************
# createPath
# Descritpion :
#	Cette fonction crée récursivement les répertoires qui n'existent pas à partir d'un chemin absolu
# Paramètres :
#	1 : chemin absolu à créer (peut être un répertoire ou un fichier)
# ****************************************
createPath ()
{
	if [ $1 != "/" ]; then
		if [ -d `dirname $1` ]; then
			if [ ! -d "$1" ]; then
				mkdir $1
				if [ $? -ne 0 ]; then
					createPathFailed=1
					return $createPathFailed
				fi
			fi
		else
			createPath `dirname $1`
			if [ "$createPathFailed" == "1" ]; then
				return $createPathFailed
			fi
			mkdir $1
			if [ $? -ne 0 ]; then
				createPathFailed=1
				return $createPathFailed
			fi
		fi
	fi
	createPathFailed=0
	return $createPathFailed
}

rollbackLevel="" # vide par défaut --> rollback n'annule rien

#### Vérification que l'utilisateur est bien "root" (on utilise cette méthode car ESX n'apas de commande "id" ni "whoami")
if [ ! $(env | grep -e "^USER=" | awk -F = '{print $2}') == "root" ]; then
	echo "Vous devez être root pour lancer ce script."
	exit 99
fi

CURRENTDIR=`pwd`
TMPDIR=/tmp/beemo-install

# on crée la structure à mettre dans beemo.tgz dans $TMPDIR
if [ ! -d $TMPDIR ]; then
	mkdir $TMPDIR
	handleError "error" 1 "Echec de création du chemin temporaire $TMPDIR"
	rollbackLevel="$rollbackLevel"a # supprime $TMPDIR
fi

# Inclusion des fichiers requis
for FILE in "/usr/local/dsr/dsr" "/etc/init.d/dsr" "/usr/local/dsr/scripts/backup_config.sh"
do
	if [ ! -d $FILE ]; then
		if [ ! -d $TMPDIR`dirname $FILE` ]; then
			createPath $TMPDIR`dirname $FILE`
			if [ "$createPathFailed" != "0" ]; then
				handleError "error" 2 "Echec de création de la structure de beemo.tgz" $rollbackLevel "force" $TMPDIR
			fi
		fi
		if [ -f $TMPDIR$FILE ]; then
			rm $TMPDIR$FILE
		fi
		cp $FILE $TMPDIR`dirname $FILE`
		handleError "error" 2 "Echec de création de la structure de beemo.tgz" $rollbackLevel "" $TMPDIR
	else
		createPath $FILE
		if [ "$createPathFailed" != "0" ]; then
			handleError "error" 2 "Echec de création de la structure de beemo.tgz" $rollbackLevel "force" $TMPDIR
		fi
	fi
done

if [ -f /usr/local/dsr/conf/config ]; then
	LICENCE=`cat /usr/local/dsr/conf/config | grep "^server=" | awk -F" +" '{print $2}' | sed 's/\r//g'`
fi

if [ "$LICENCE" == "" ]; then
	LICENCE="null"
fi

# Inclusion des fichiers facultatifs
for FILE in "/usr/local/dsr/conf/config" "/usr/local/dsr/conf/pub" "/usr/local/dsr/conf/priv" "/usr/local/dsr/conf/$LICENCE/pub" "/usr/local/dsr/plugins/esx/config"
do
	if [ -f $FILE ]; then
		if [ ! -d $FILE ]; then
			if [ ! -d $TMPDIR`dirname $FILE` ]; then
				createPath $TMPDIR`dirname $FILE`
				if [ "$createPathFailed" != "0" ]; then
					handleError "warning" 7 "Impossible d'inclure $FILE lors de la création de la structure de beemo.tgz" "" "force"
				fi
			fi
			if [ -f $TMPDIR$FILE ]; then
				rm $TMPDIR$FILE
			fi
			cp $FILE $TMPDIR`dirname $FILE`
			handleError "warning" 7 "Impossible d'inclure $FILE lors de la création de la structure de beemo.tgz" "" ""
		else
			createPath $FILE
			if [ "$createPathFailed" != "0" ]; then
				handleError "warning" 7 "Impossible d'inclure $FILE lors de la création de la structure de beemo.tgz" "" "force"
			fi
		fi
	fi
done

# On lève un warning si $LICENCE n'est pas null et que l'inclusion de /usr/local/dsr/conf/$LICENCE/pub échoue
if [ "$LICENCE" != "null" ] && [ ! -f $TMPDIR/usr/local/dsr/conf/$LICENCE/pub ]; then
	handleError "warning" 8 "Licence non null dans fichier config mais fichier /usr/local/dsr/conf/$LICENCE/pub n'a pu être inclus à beemo.tgz" "" "force"
fi

if [ ! -d $TMPDIR/etc/init.d/ ]; then
	createPath $TMPDIR/etc/init.d/
	if [ "$createPathFailed" != "0" ]; then
		handleError "error" 2 "Echec de création de la structure de beemo.tgz" $rollbackLevel "" $TMPDIR
	fi
fi
if [ -f $TMPDIR/etc/init.d/dsr ]; then
	rm $TMPDIR/etc/init.d/dsr
fi
cp /etc/init.d/dsr $TMPDIR/etc/init.d/
handleError "error" 2 "Echec de création de la structure de beemo.tgz" $rollbackLevel "" $TMPDIR

## Ajout du fichier d'authorisation du bind dans ESXi 5.1+
if [ -f /etc/vmware/secpolicy/tardisks/dsr ]; then
	createPath "$TMPDIR/etc/vmware/secpolicy/tardisks/"
	if [ "$createPathFailed" != "0" ]; then
		handleError "error" 2 "Echec de création de la structure de beemo.tgz" $rollbackLevel "force" $TMPDIR
	fi
	if [ -f "$TMPDIR/etc/vmware/secpolicy/tardisks/dsr" ]; then
		rm "$TMPDIR/etc/vmware/secpolicy/tardisks/dsr"
	fi
	cp /etc/vmware/secpolicy/tardisks/dsr "$TMPDIR/etc/vmware/secpolicy/tardisks/"
	handleError "error" 2 "Echec de création de la structure de beemo.tgz" $rollbackLevel "" $TMPDIR
fi

# on crée beemo.tgz à partir de $TMPDIR et on le copie dans /bootbank
cd $TMPDIR

chmod -R +t *
handleError "error" 2 "Echec de création de la structure de beemo.tgz" $rollbackLevel "" $TMPDIR
tar -czf beemo.tgz *
handleError "error" 3 "Echec de création de beemo.tgz" $rollbackLevel "" $TMPDIR

mv beemo.tgz /bootbank/
handleError "error" 4 "Echec de la copie de beemo.tgz dans /bootbank" $rollbackLevel "" $TMPDIR
rollbackLevel="$rollbackLevel"b # supprime /bootbank/beemo.tgz

# on ajoute beemo.tgz à la ligne modules de boot.cfg s'il n'y est pas déjà
IS_BEE_MODULE=`grep beemo.tgz /bootbank/boot.cfg`
GREP_RETURN=$?
if [ $GREP_RETURN -ne 0 ] && [ $GREP_RETURN -ne 1 ]; then
	handleError "error" 51 "Echec de l'ajout du module beemo.tgz à boot.cfg. Impossible de vérifier la présence antérieure du module." $rollbackLevel "force" $TMPDIR
fi
if [ "$IS_BEE_MODULE" == "" ]; then
	if [ -f /bootbank/boot.cfg.beemoinstallbak ]; then
		rm /bootbank/boot.cfg.beemoinstallbak
	fi
	cp /bootbank/boot.cfg /bootbank/boot.cfg.beemoinstallbak
	sed -e '/modules=/s/$/ --- beemo.tgz/' -i /bootbank/boot.cfg
	handleError "error" 5 "Echec de l'ajout du module beemo.tgz à boot.cfg" $rollbackLevel "" $TMPDIR
	rollbackLevel="$rollbackLevel"c # restore /bootbank/beemo.tgz avec /bootbank/boot.cfg.beemoinstallbak
else
	echo "Module beemo déjà présent dans boot.cfg"
fi

cd $CURRENTDIR
rm -Rf $TMPDIR
handleError "warning" 6 "Echec de la suppression du repertoire temporaire :\n $TMPDIR\nVous pouvez le supprimer manuellement, sinon il dera détruit au prochain reboot."
